﻿using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace IrcTokens.Tests
{
    [TestClass]
    public class Hostmask
    {
        [TestMethod]
        public void TestHostmask()
        {
            var hostmask = new IrcTokens.Hostmask("nick!user@host");
            Assert.AreEqual("nick", hostmask.NickName);
            Assert.AreEqual("user", hostmask.UserName);
            Assert.AreEqual("host", hostmask.HostName);
        }

        [TestMethod]
        public void TestNoHostName()
        {
            var hostmask = new IrcTokens.Hostmask("nick!user");
            Assert.AreEqual("nick", hostmask.NickName);
            Assert.AreEqual("user", hostmask.UserName);
            Assert.IsNull(hostmask.HostName);
        }

        [TestMethod]
        public void TestNoUserName()
        {
            var hostmask = new IrcTokens.Hostmask("nick@host");
            Assert.AreEqual("nick", hostmask.NickName);
            Assert.IsNull(hostmask.UserName);
            Assert.AreEqual("host", hostmask.HostName);
        }

        [TestMethod]
        public void TestOnlyNickName()
        {
            var hostmask = new IrcTokens.Hostmask("nick");
            Assert.AreEqual("nick", hostmask.NickName);
            Assert.IsNull(hostmask.UserName);
            Assert.IsNull(hostmask.HostName);
        }

        [TestMethod]
        public void TestHostmaskFromLine()
        {
            var line     = new Line(":nick!user@host PRIVMSG #channel hello");
            var hostmask = new IrcTokens.Hostmask("nick!user@host");
            Assert.AreEqual(hostmask.ToString(), line.Hostmask.ToString());
            Assert.AreEqual("nick", line.Hostmask.NickName);
            Assert.AreEqual("user", line.Hostmask.UserName);
            Assert.AreEqual("host", line.Hostmask.HostName);
        }

        [TestMethod]
        public void TestEmptyHostmaskFromLine()
        {
            var line = new Line("PRIVMSG #channel hello");
            Assert.IsNull(line.Hostmask.HostName);
            Assert.IsNull(line.Hostmask.UserName);
            Assert.IsNull(line.Hostmask.NickName);
        }
    }
}
